#!/usr/bin/perl
##############################################################################
#	Procedure:	psuedo_MLM.pl
#	Version:	V3.1-01
#	Author:		Gary Boyles
#	Date:		31-Jan-2000		12:00
#
# Description:
# ===========
# Emulate or mimic an MLM by taking an "Interface_Up" and/or 
# Interface_Down" event and forwarding to another system as an MLM-generated
# interface-up/down event.
#
# Modifications:
#
# Date		Who	Reason
# ===========	===	======================================================
# 05-Jan-2000	gpb	Changed usage keyword			      V3.1-01
# 12-Oct-1999	gpb	Version sync				      V3.0-05
# 24-Sep-1999	gpb	Converted TCL code to enhanced perl	      V3.0-01
##############################################################################

$prgName	= 'psuedo_MLM';
$prgFile	= 'psuedo_MLM.pl';
$prgVer		= 'V3.1-01';

#++++++++++
# Initialize Global Variables
######################################################################
$True		= "1";			# Perl standard for true/false
$False		= "0";
@rtnList	= ();
@address	= ();
$dx		= "";
$dy		= "";
$dz		= "";

#++++++++++
# Get arguments
##############################################################################
$thisNode	= "$ARGV[0]";	# Node trap applies to
$thisAddr	= "$ARGV[1]";	# Addr trap is specifically for
$thisStat	= "$ARGV[2]";	# Status Up/Down (or) NV 58916866/58916867.

#++++++++++
# Check to see if we need help
##############################################################################
if ( "$ARGV[0]" eq "-h" || $ARGV[0] eq "-help" || $#ARGV < 2)
  { $helpFlag = "$True"; }
else
  { $helpFlag = "$False"; }

#++++++++++
# Print out usage message
##############################################################################
if ( "$helpFlag" eq "$True" )
  {
  print "Usage:    $prgFile  node addr status \n\n";
  print "e.g. $prgFile abx.xyz.xxx.com 10.10.1.1 up/normal \n\n";
  print "e.g. $prgFile abx.xyz.xxx.com 10.10.1.1 down/critical \n\n";
  print "e.g. $prgFile \$2 \"\$4\" \$S \n\n";
  print "Config_File:  $prgName.conf \n";
  print "Log_File:     $prgName.log \n";
  print "Directories: \n";
  print "    \\usr\\OV\\conf\\   (Win-NT) \n";
  print "    \\usr\\OV\\log\\ \n";
  print "    /usr/OV/conf/   (Unix) \n";
  print "    /usr/OV/log/    \n\n";
  exit(0);
  }

#++++++++++
# Begin main() routine
##############################################################################
print ("\n$prgName $prgVer  ($thisNode)\n\n");

#++++++++++
# Parse and resolve the node-name
##############################################################################
$_ = "$thisNode";		# Get and resolve node-name
~s/^\s*//;
s/\s*$//;
@rtnList = "$_";
$thisNode = "$rtnList[0]";	# Assume 1st word is node or addr

($thisNode, $dx,$dy,$dz, @address) = gethostbyname("$thisNode");
				# Perform name-resolution
#++++++++++
# Parse the IP-address
##############################################################################
$_ = "$thisAddr";		# Get and resolve node-name
~s/^\s*//;
s/\s*$//;
@rtnList  = "$_";
$thisAddr = "$rtnList[0]";	# Assume 1st word is node or addr

#++++++++++
# Resolve status
##############################################################################
$_ = "$thisStat";		# Get and resolve node-name
tr/A-Z/a-z/;
$thisStat = "$_";
if ( "$thisStat" eq "58916866" || "$thisStat" eq "up" || "$thisStat" eq "normal")
  { $thisStat = "up"; }
elsif ("$thisStat" eq "58916867" || "$thisStat" eq "down" || "$thisStat" eq "critical")
  { $thisStat = "down"; }
else
  { $helpFlag = "$True"; }

#++++++++++
# Log the MLM event
##############################################################################
&log_msg( "$thisNode", "$thisAddr", "$thisStat" );

#++++++++++
# Send the MLM traps to the NetView SNMP-mgmt stations defined in config-file
##############################################################################

&send_MLM_traps( "$thisNode", "$thisAddr", "$thisStat");

exit(0);				# Exit the program



##############################################################################
# Subroutine:   log_msg
# Input:        node, address, status
# Outputs:      Log Entry to log-file
# Description:  Log an entry to the specified log-file
##############################################################################
sub log_msg
{
  my ($thePad)	= "                                        ";
  my ($the_OS)	= $ENV{"OS"};
  my ($timeStamp) = &get_timeStamp;
  my ($logFile)	= "";
  my ($dPtr)	= 0;
  my ($theNode)	= $_[0];
  my ($theAddr)	= $_[1];
  my ($theStat)	= $_[2];

if ("$prgName" eq "") { $prgName = "psuedo_MLM"; }

if ("$the_OS" eq "Windows_NT")
  {
  $nv_drive	= $ENV{"NV_DRIVE"};
  $logFile	= "$nv_drive"."\\usr\\OV\\log\\"."$prgName".".log";
  }
else
  { $logFile	= "/usr/OV/log/"."$prgName".".log"; }

$theNode  = "$theNode"."$thePad";
$theNode  = substr("$theNode",0, 40 );

$theAddr  = "$theAddr"."$thePad";
$theAddr  = substr("$theAddr",0, 15 );

$theStat  = "$theStat"."$thePad";
$theStat  = substr("$theStat",0,4 );

open  (LOG_FILE, ">> $logFile");
  print (LOG_FILE "$timeStamp $theNode $theAddr $theStat \n");
close (LOG_FILE);

}

##############################################################################
# Subroutine:	get_timeStamp
# Input:	None
# Outputs:	timestamp in format of mm/dd/yyyy hh:mm
# Description:	Return the date+time for today.
##############################################################################
sub get_timeStamp
{
  my ($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $isDst);
  my ($theDate, $theTime);

( $sec, $min, $hour, $mday, $month, $year, $wday, $yday, $isDst ) =
        localtime(time);

$month++;
if ( length ($month) < 2 ) { $month = "0"."$month"; } # 2-digit month
if ( length ($mday)  < 2 ) { $mday  = "0"."$mday"; }  # 2-digit day
if ( length ($year)  < 4 ) { $year = 1900 + $year; }  # Year-2000 code
if ( length ($hour) == 1 ) { $hour = "0"."$hour"; }
if ( length ($min)  == 1 ) { $min  = "0"."$min"; }

$theDate = "$month"."/"."$mday"."/"."$year";
$theTime = "$hour".":"."$min";

("$theDate $theTime");

}

##############################################################################
# Subroutine:   send_MLM_traps
# Input:        node, interface-address, status-of-interface
# Outputs:      None
# Description:
# Send an snmptrap which mimics an MLM interface-up or MLM interface-down.
# This should work on NT (tested) or UNIX.
##############################################################################
sub send_MLM_traps
{
  my ($the_OS)	= $ENV{"OS"};
  my ($Eprise)	= ".1.3.6.1.4.1.2.6.12";
  my ($pdu_1)	= ".1.3.6.1.4.1.2.6.12.3.2.1.1.1 Integer";
  my ($pdu_2)	= ".1.3.6.1.4.1.2.6.12.3.2.1.1.2 OctetString";
  my ($pdu_3)	= ".1.3.6.1.4.1.2.6.12.3.2.1.1.3 OctetString";
  my ($pdu_4)	= ".1.3.6.1.4.1.2.6.12.3.2.1.1.4 OctetString";
  my ($MLM_iDown)	= "11";
  my ($MLM_iUp)	= "12";
  my ($rawAddr)	= "0x00_00_00_00";
  my ($cs)		= "public";	# Community-string
  my ($all_PDUs)	= "";
  my ($local_host)	= "";
  my ($IP_Agent)	= "";
  my ($nv_drive)	= "";
  my ($configFile)	= "";
  my ($send_trap)	= "";
  my (@address)	= ();
  my (@pLine)	= ();
  my ($dx,$dy,$dz);
  my ($snmpMgr)	= "";
  my ($theNode)	= $_[0];	# Node trap pertains to
  my ($theAddr)	= $_[1];	# Address trap pertains to
  my ($theStat)	= $_[2];	# Status = up/down

if ("$prgName" eq "") { $prgName = "psuedo_MLM"; }

#++++++++++
# Setup the variables depending on the OS we're running on
##############################################################################
if ("$the_OS" eq "Windows_NT")
  {
  $local_host	= $ENV{"COMPUTERNAME"};
  $nv_drive	= $ENV{"NV_DRIVE"};
  $send_trap	= "$nv_drive"."\\usr\\OV\\bin\\nvsnmptrap.exe";
  $configFile	= "$nv_drive"."\\usr\\OV\\conf\\"."$prgName".".conf";
  }
else
  {
  $local_host	= `hostname`;
  $send_trap	= "/usr/OV/bin/snmptrap";
  $configFile	= "/usr/OV/conf/"."$prgName".".conf";
  }

#++++++++++
# Get fully-qualified node-name to use as agent in trap
##############################################################################
($IP_Agent, $dx,$dy,$dz, @address) = gethostbyname("$local_host");

#++++++++++
# Configure all the PDUs, determine the specific-trap.
##############################################################################
$all_PDUs = "$pdu_1 2 $pdu_2 $rawAddr $pdu_3 $theAddr $pdu_4 $theNode";

if ("$theStat" eq "down")
  { $sTrap = $MLM_iDown; }
else
  { $sTrap = $MLM_iUp; }

#++++++++++
# Read the config-file.  Get the list of target-nodes & community-strings
##############################################################################
if ( -r $configFile )				# Make sure we can access file
  {
  open(CONFIG_FILE, "$configFile");		# Open the file

  while ( $inLine = <CONFIG_FILE>)		# Read file line-by-line
    {
    chop($inLine);
    if ( "$inLine" ne "" && $inLine!~/#/)	# Skip blanks & comments
      {
      @pLine	= split(/\s+/,"$inLine");	# Split line into components
      $snmpMgr	= "$pLine[0]";			# Get target NetView system
      $cs	= "$pLine[1]";			# Get community-string
      if ("$cs" eq "") { $cs = "public"; }	# Add default if necessary

      print "\n$send_trap -c $cs $snmpMgr $Eprise $IP_Agent 6 $sTrap 0 $all_PDUs \n\n";

      $dx = `$send_trap -c $cs $snmpMgr $Eprise $IP_Agent 6 $sTrap 0 $all_PDUs`;
      }						# Send the MLM trap
    }
  }
else
  {
  print "\n$prgName -- Aborting !!!\n\n";
  print "Cannot read config-file -- $configFile \n\n";
  print "Create File With Format:  snmpMgr  community-string \n";
  print "  1)  one mgr-target node per line (as many targets as you want). \n";
  print "  2)  community-string is optional (\"public\" assumed). \n";
  print "  3)  Comments at line-beginning allowed... start with \"\#\". \n\n";
  exit (0);
  }

}

1;
